function [Z,EstR,Estw] = L1MC(X,Omega,mu, maxIter,InitialR)
% L1MC aims to estimate the rank of incomplete matrix
%minimize  \mu||w||_1 + 1/2* ||X_Z||_F^2, where Z= w_1*u_1*v_1' +.... w_r*u_r*v_r'+.... w_R*u_R*v_R'. u and v are unit vectors.
% InitialR is the matrix rank which can be automatically estimated;
%% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
%%% Input:
%     X ---  The incomplete matrix whose elements in Omega are used for estimating missing value;
%     Omega ---  The index set of the obeserved elements: known 1, unkown, 0;
%     mu --- The parameter of the L1-norm regularzation term;
%     maxIter ---  The maximum iterations;
%     epsilon ---  The stop tolerance of the error of approximation Z and input X on observed entries;
%     InitialR  --- The initial maximum rank for target matrix);

%%% Output:
% Z --- Recovered matrix
% EstR --- Estimated Rank
% Estw --- Estimated weight vector (singular values)
%%% Last Update on 05/17/2016. Copyright by Qiquan SHI

% Initialization
DimX = ndims(X);
sizeX = size(X);
X(~Omega)= mean(mean(X));  % Set the missing elements as 0, you also can try mean of observed elements;
fr=size(find(Omega==1),1)/prod(sizeX) * 1e-3
w=rand(InitialR,1);
for i = 1 : DimX
    U0{i} = rand(sizeX(i),InitialR);
    temp=sqrt(sum(U0{i}.^2,1));
    U0{i}= U0{i}./((ones(size(U0{i},1),1))*temp);
end
U=U0{1};
V=U0{2};
Xold=0;
Z=zeros(sizeX);
%% Iteration Scheme

for k = 1: maxIter
    Xr =  X;
    for  r = 1 : InitialR
        if w(r) ~= 0
            % Update u_r and v_r
            U(:,r) =(Xr*V(:,r))/ w(r);
            U(:,r) = U(:,r) /norm(U(:,r));
            
            V(:,r) = (Xr'*U(:,r)) /w(r);
            V(:,r) = V(:,r) /norm(V(:,r));
            
            % Update w_r
            midw= U(:,r)*V(:,r)';
            S =dot(Xr(:),midw(:));
            %                 S =trace(Xr*midw');
            w(r) = soft(S,mu); %  Using L1-norm regularzation to sparse weights then estimate rank
            
            Xr =  Xr - w(r)*midw; % Residual of approximation
        end
    end
    
    Z =  X -Xr;            % Approximation Z aims to finally equal to ground truth M;
    X(~Omega) = Z(~Omega); % Update the missing part in X using approximated vaules in Z;
    
    %% Check the stop criteria
    stopC=0;
    error= X(Omega) - Z(Omega);
    stopC = max(stopC, norm(error(:))/norm(X(Omega)));
    ErrList(k) = stopC;
    X_ErrList(k) =  norm (X-Xold,'fro')/ norm(X,'fro');
    if mod(k, 200) == 0
        fprintf('\n----------L1MC E3: iterations = %d, **** difference = %5d--X_ErrList = %5d---\n', k, ErrList(k), X_ErrList(k) );
    end
    epsilon= 1e-4;
    if  X_ErrList(k)  <  epsilon || stopC < 1e-4
        %                 fprintf('\n----------L1MC-RF-R1MC Stop: iterations = %d, L= %d, **** difference = %5d--X_ErrList Error= %5d---\n', k,l, ErrList(k,l), X_ErrList(k,l) );
        %% Estimate the rank by throwing away the very small weights
        if find(w==0)
            R=length(find(w/sum(w)>fr)); % Estimate the real rank of the incomplete matrix X
            InitialR=R;
            fprintf('******** L1MC estimated rank = %d, iterations = %d******** \n \n ', InitialR,k);
            index=find(w/sum(w)>fr);
            U0=zeros(sizeX(1),InitialR);
            V0=zeros(sizeX(2),InitialR);
            for tt=1:InitialR
                U0(:,tt)=U(:,index(tt)); % Keep the correspoding factor vectors U
                V0(:,tt)=V(:,index(tt)); % Keep the correspoding factor vectors V
            end
            U=U0;
            V=V0;
            w=w(index);
            Z=U*diag(w)*V';
            X(~Omega) = Z(~Omega);
            break;
        end
        if InitialR==0
            break;
        end
    end
    Xold=X;
    
end

%% Estimate the rank of the incomplete matrix X
index=find(w/sum(w)>fr);
R=length(index);
InitialR=R;
fprintf('******** L1MC estimated rank = %d, iterations = %d******** \n \n ', InitialR,k);
U0=zeros(sizeX(1),InitialR);
V0=zeros(sizeX(2),InitialR);
for tt=1:InitialR
    U0(:,tt)=U(:,index(tt)); % Keep the correspoding factor vectors U
    V0(:,tt)=V(:,index(tt)); % Keep the correspoding factor vectors V
end
U=U0;
V=V0;
w=w(index);
Z=U*diag(w)*V';
X(~Omega) = Z(~Omega);


EstR=R;
Estw=w;
% Soft thresholding
    function y = soft(x,T)
        if sum(abs(T(:)))==0
            y = x;
        else
            y = max(abs(x) - T, 0);
            y = sign(x).*y;
        end
    end

end